import re
from chat_exporter.emoji_convert import convert_emoji


async def parse_emoji(content):
    content = re.sub(r"\n", "<br>", content)
    output = []
    for word in content.split():
        emoji_pattern = re.compile(r"&lt;:.*:.*&gt;")
        emoji_animated = re.compile(r"&lt;a:.*:.*&gt;")
        if emoji_pattern.search(word):
            pattern = r"&lt;:.*:(\d*)&gt;"
            emoji_id = re.search(pattern, word).group(1)
            new_w = f'<img class="emoji emoji--small" src="https://cdn.discordapp.com/emojis/{str(emoji_id)}.png">'
            word = re.sub(emoji_pattern, new_w, word)
            output.append(word)
        elif emoji_animated.search(word):
            pattern = r"&lt;a:.*:(\d*)&gt;"
            emoji_id = re.search(pattern, word).group(1)
            new_w = f'<img class="emoji emoji--small" src="https://cdn.discordapp.com/emojis/{str(emoji_id)}.gif">'
            word = re.sub(emoji_animated, new_w, word)
            output.append(word)
        else:
            word = convert_emoji(word)
            output.append(word)
    content = " ".join(output)
    return content


async def parse_embed_markdown(content):
    # [Message](Link)

    pattern = re.compile(r'<div class="chatlog__embed-field-value"><span class="markdown">(.+)?</span></div>')
    match = re.search(pattern, content)
    if match is not None:
        field_value = match.group(1)
        pattern = re.compile(r"\[(.+)?]\((.+)?\)")
        match = re.search(pattern, field_value)
        change = False
        while match is not None:
            change = True
            affected_text = match.group(1)
            affected_url = match.group(2)
            field_value = field_value.replace(field_value[match.start():match.end()],
                                  '<a href="%s">%s</a>' % (affected_url, affected_text))
            match = re.search(pattern, field_value)

        if change:
            pattern = re.compile(r'<div class="chatlog__embed-field-value"><span class="markdown">(.+)?</span></div>')
            match = re.search(pattern, content)
            content = content.replace(content[match.start():match.end()],
                                      '<div class="chatlog__embed-field-value"><span class="markdown">%s</span></div>'
                                      % field_value)

    if match is None:
        pattern = re.compile(r"\[(.+)?]\((.+)?\)")
        match = re.search(pattern, content)
        while match is not None:
            affected_text = match.group(1)
            affected_url = match.group(2)
            content = content.replace(content[match.start():match.end()],
                                      '<a href="%s">%s</a>' % (affected_url, affected_text))
            match = re.search(pattern, content)

    return content


async def parse_markdown(content):
    # **bold**
    pattern = re.compile(r"\*\*(.*?)\*\*")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '<strong>%s</strong>' % affected_text)
        match = re.search(pattern, content)

    # *italic*
    pattern = re.compile(r"\*(.*?)\*")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '<em>%s</em>' % affected_text)
        match = re.search(pattern, content)

    # __underline__
    pattern = re.compile(r"__(.*?)__")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '<span style="text-decoration: underline">%s</span>' % affected_text)
        match = re.search(pattern, content)

    # ~~strikethrough~~
    pattern = re.compile(r"~~(.*?)~~")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '<span style="text-decoration: line-through">%s</span>' % affected_text)
        match = re.search(pattern, content)

    # ||spoiler||
    pattern = re.compile(r"\|\|(.*?)\|\|")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '<span class="spoiler spoiler--hidden" onclick="showSpoiler(event, this)">'
                                  '<span class="spoiler-text">%s</span></span>' % affected_text)
        match = re.search(pattern, content)

    # > quote
    pattern = re.compile(r"^&gt;\s(.+)")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        br_pattern = re.compile(r"^&gt;\s(.+?)<br>")
        if re.search(br_pattern, content):
            match = re.search(br_pattern, content)
            affected_text = match.group(1)
            content = content.replace(content[match.start():match.end()],
                                      '<div class="quote">%s</div>' % affected_text)
        else:
            content = content.replace(content[match.start():match.end()],
                                      '<div class="quote">%s</div>' % affected_text)
        match = re.search(pattern, content)

    pattern = re.compile(r"<br>&gt;\s(.+)")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        br_pattern = re.compile(r"<br>&gt;\s(.+?)<br>")
        if re.search(br_pattern, content):
            match = re.search(br_pattern, content)
            affected_text = match.group(1)
            content = content.replace(content[match.start():match.end()],
                                      '<div class="quote">%s</div>' % affected_text)
        else:
            content = content.replace(content[match.start():match.end()],
                                      '<div class="quote">%s</div>' % affected_text)
        match = re.search(pattern, content)

    # ```code```
    pattern = re.compile(r"```(.*?)```")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        if affected_text.lower().startswith(("asciidoc", "autohotkey", "bash", "coffeescript", "cpp", "cs", "css",
                                             "diff", "fix", "glsl", "ini", "json", "md", "ml", "prolog", "py",
                                             "tex", "xl", "xml")):
            affected_text = affected_text.replace("<br>", " <br>")
            affected_text = ' '.join(affected_text.split()[1:])
        affected_text = re.sub(r"^<br>", "", affected_text)
        affected_text = re.sub(r"<br>$", "", affected_text)
        affected_text = await return_to_markdown(affected_text)
        content = content.replace(content[match.start():match.end()],
                                  '<div class="pre pre--multiline nohighlight">%s</div>' % affected_text)
        match = re.search(pattern, content)

    # ``code``
    pattern = re.compile(r"``(.*?)``")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        affected_text = await return_to_markdown(affected_text)
        content = content.replace(content[match.start():match.end()],
                                  '<span class="pre pre-inline">%s</span>' % affected_text)
        match = re.search(pattern, content)

    # `code`
    pattern = re.compile(r"`(.*?)`")
    match = re.search(pattern, content)
    while match is not None:
        affected_text = match.group(1)
        affected_text = await return_to_markdown(affected_text)
        content = content.replace(content[match.start():match.end()],
                                  '<span class="pre pre-inline">%s</span>' % affected_text)
        match = re.search(pattern, content)

    return content


spoiler = r''


async def return_to_markdown(content):
    # content = re.sub(r"^<br>", "", content)

    for match in re.finditer(r"<strong>(.*?)</strong>", content):
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '**%s**' % affected_text)

    for match in re.finditer(r"<em>([^<>]+)</em>", content):
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '*%s*' % affected_text)

    for match in re.finditer(r'<span style="text-decoration: underline">([^<>]+)</span>', content):
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '__%s__' % affected_text)

    for match in re.finditer(r'<span style="text-decoration: line-through">([^<>]+)</span>', content):
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '~~%s~~' % affected_text)

    for match in re.finditer(r'<span class="spoiler spoiler--hidden" onclick="showSpoiler\(event, this\)">'
                             r'<span class="spoiler-text">(.*?)</span></span>', content):
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '||%s||' % affected_text)

    for match in re.finditer(r'<div class="quote">(.*?)</div>', content):
        affected_text = match.group(1)
        content = content.replace(content[match.start():match.end()],
                                  '> %s' % affected_text)

    return content



import html


async def escape_html(content):
    return html.escape(content)


async def member_colour_translator(member):
    r_colour = 0
    g_colour = 0
    b_colour = 0
    try:
        for roles in member.roles:
            if roles.colour.r != 0:
                r_colour = roles.colour.r
                g_colour = roles.colour.g
                b_colour = roles.colour.b
            if roles.colour.g != 0:
                r_colour = roles.colour.r
                g_colour = roles.colour.g
                b_colour = roles.colour.b
            if roles.colour.b != 0:
                r_colour = roles.colour.r
                g_colour = roles.colour.g
                b_colour = roles.colour.b
    except AttributeError:
        pass
    if r_colour == 0 and g_colour == 0 and b_colour == 0:
        r_colour = 255
        g_colour = 255
        b_colour = 255
    user_colour = f"color: #%02x%02x%02x;" % (r_colour, g_colour, b_colour)
    return user_colour



import re
from chat_exporter.misc_tools import escape_html

REGEX_ROLES = r"&lt;@&amp;([0-9]+)&gt;"
REGEX_MEMBERS = r"&lt;@!?([0-9]+)&gt;"
REGEX_CHANNELS = r"&lt;#([0-9]+)&gt;"
REGEX_EMOJIS = r"&lt;a?(:[^\n:]+:)[0-9]+&gt;"
REGEX_ROLES_2 = r"<@&([0-9]+)>"
REGEX_MEMBERS_2 = r"<@!?([0-9]+)>"
REGEX_CHANNELS_2 = r"<#([0-9]+)>"
REGEX_EMOJIS_2 = r"<a?(:[^\n:]+:)[0-9]+>"

ESCAPE_LT = "______lt______"
ESCAPE_GT = "______gt______"
ESCAPE_AMP = "______amp______"


async def escape_mentions(content):
    for match in re.finditer("(%s|%s|%s|%s|%s|%s|%s|%s)"
                             % (REGEX_ROLES, REGEX_MEMBERS, REGEX_CHANNELS,
                                REGEX_EMOJIS, REGEX_EMOJIS_2, REGEX_ROLES_2, REGEX_CHANNELS_2, REGEX_MEMBERS_2),
                             content):

        pre_content = content[:match.start()]
        post_content = content[match.end():]
        match_content = content[match.start():match.end()]

        match_content = match_content.replace("<", ESCAPE_LT)
        match_content = match_content.replace(">", ESCAPE_GT)
        match_content = match_content.replace("&", ESCAPE_AMP)

        content = pre_content + match_content + post_content

    return content


async def unescape_mentions(content):
    content = content.replace(ESCAPE_LT, "<")
    content = content.replace(ESCAPE_GT, ">")
    content = content.replace(ESCAPE_AMP, "&")
    return content


async def parse_mentions(content, guild, bot):
    # parse mentions
    # channels
    offset = 0
    for match in re.finditer(REGEX_CHANNELS, content):
        channel_id = int(match.group(1))
        channel = guild.get_channel(channel_id)
        if channel is None:
            replacement = '#deleted-channel'
        else:
            replacement = '<span class="mention" title="%s">#%s</span>' \
                      % (channel.name, channel.name)
        content = content.replace(content[match.start() + offset:match.end() + offset],
                                  replacement)
        offset += len(replacement) - (match.end() - match.start())

    for match in re.finditer(REGEX_CHANNELS_2, content):
        channel_id = int(match.group(1))
        channel = guild.get_channel(channel_id)
        if channel is None:
            replacement = '#deleted-channel'
        else:
            replacement = '<span class="mention" title="%s">#%s</span>' \
                      % (channel.name, channel.name)
        content = content.replace(content[match.start() + offset:match.end() + offset],
                                  replacement)
        offset += len(replacement) - (match.end() - match.start())
    # roles
    offset = 0
    for match in re.finditer(REGEX_ROLES, content):
        role_id = int(match.group(1))
        role = guild.get_role(role_id)
        if role is None:
            replacement = '@deleted-role'
        else:
            replacement = '<span style="color: #%02x%02x%02x;">@%s</span>' \
                          % (role.color.r, role.color.g, role.color.b, role.name)
        content = content.replace(content[match.start() + offset:match.end() + offset], replacement)
        offset += len(replacement) - (match.end() - match.start())

    for match in re.finditer(REGEX_ROLES_2, content):
        role_id = int(match.group(1))
        role = guild.get_role(role_id)
        if role is None:
            replacement = '@deleted-role'
        else:
            replacement = '<span style="color: #%02x%02x%02x;">@%s</span>' \
                          % (role.color.r, role.color.g, role.color.b, role.name)
        content = content.replace(content[match.start() + offset:match.end() + offset], replacement)
        offset += len(replacement) - (match.end() - match.start())

    # members
    offset = 0

    for match in re.finditer(REGEX_MEMBERS, content):
        member_id = int(match.group(1))
        member = guild.get_member(member_id) or bot.get_user(member_id)

        try:
            member_name = await escape_html(member.nick)
        except AttributeError:
            member_name = await escape_html(member.name)

        if member is not None:
            replacement = '<span class="mention" title="%s">@%s</span>' \
                          % (member, str(member_name))
            content = content.replace(content[match.start() + offset:match.end() + offset],
                                      replacement)
            offset += len(replacement) - (match.end() - match.start())

    for match in re.finditer(REGEX_MEMBERS_2, content):
        member_id = int(match.group(1))
        member = guild.get_member(member_id) or bot.get_user(member_id)

        try:
            member_name = await escape_html(member.nick)
        except AttributeError:
            member_name = await escape_html(member.name)

        if member is not None:
            replacement = '<span class="mention" title="%s">@%s</span>' \
                          % (member, str(member_name))
            content = content.replace(content[match.start() + offset:match.end() + offset],
                                      replacement)
            offset += len(replacement) - (match.end() - match.start())

    # custom emoji
    offset = 0
    for match in re.finditer(REGEX_EMOJIS, content):
        name = match.group(1)
        replacement = name
        content = content.replace(content[match.start() + offset:match.end() + offset],
                                  replacement)
        offset += len(replacement) - (match.end() - match.start())

    for match in re.finditer(REGEX_EMOJIS_2, content):
        name = match.group(1)
        replacement = name
        content = content.replace(content[match.start() + offset:match.end() + offset],
                                  replacement)
        offset += len(replacement) - (match.end() - match.start())

    return content



import io
import re
import os
import discord
import sys
import traceback
from chat_exporter.misc_tools import escape_html, member_colour_translator
from chat_exporter.mention_convert import parse_mentions, escape_mentions, unescape_mentions
from chat_exporter.markdown_convert import parse_markdown, parse_embed_markdown, parse_emoji
from chat_exporter.emoji_convert import convert_emoji
from pytz import timezone
from datetime import timedelta

dir_path = os.path.dirname(os.path.realpath(__file__))

eastern = timezone("US/Eastern")
utc = timezone("UTC")

bot = None


def init_exporter(_bot):
    global bot
    bot = _bot


async def export(ctx):
    # noinspection PyBroadException
    try:
        transcript = await produce_transcript(ctx.channel)
    except Exception:
        transcript = None
        print("Error during transcript generation!", file=sys.stderr)
        traceback.print_exc()
        error_embed = discord.Embed(
            title="Transcript Generation Failed!",
            description="Whoops! We've stumbled in to an issue here.",
            colour=discord.Colour.red()
        )
        await ctx.channel.send(embed=error_embed)
        print(f"Please send a screenshot of the above error to https://www.github.com/mahtoid/DiscordChatExporterPy")

    if transcript is not None:
        async for m in ctx.channel.history(limit=None):
            try:
                for f in m.attachments:
                    if f"transcript-{ctx.channel.name}.html" in f.filename:
                        await m.delete()
            except TypeError:
                continue

        # Save transcript
        transcript_embed = discord.Embed(
            description=f"**Transcript Name:** transcript-{ctx.channel.name}\n\n"
                        f"{ctx.author.mention} requested a transcript of the channel",
            colour=discord.Colour.blurple()
        )

        transcript_file = discord.File(io.BytesIO(transcript.encode()),
                                       filename=f"transcript-{ctx.channel.name}.html")

        await ctx.channel.send(embed=transcript_embed, file=transcript_file)


async def generate_transcript(channel):
    # noinspection PyBroadException
    try:
        transcript = await produce_transcript(channel)
    except Exception:
        transcript = None
        print(f"Please send a screenshot of the above error to https://www.github.com/mahtoid/DiscordChatExporterPy")

    return transcript


async def produce_transcript(channel):
    guild = channel.guild
    messages = await channel.history(limit=None, oldest_first=True).flatten()
    previous_author = 0
    previous_timestamp = ""
    messages_html = ""
    for m in messages:
        time_format = "%b %d, %Y %I:%M %p"
        time_string = utc.localize(m.created_at).astimezone(eastern)
        time_string_created = time_string.strftime(time_format)
        if m.edited_at is not None:
            time_string_edited = utc.localize(m.edited_at).astimezone(eastern)
            time_string_edited = time_string_edited.strftime(time_format)
            time_string_final = "%s (edited %s)" \
                                % (time_string_created, time_string_edited)
        else:
            time_string_final = time_string_created

        m.content = await escape_html(m.content)
        m.content = re.sub(r"\n", "<br>", m.content)

        embeds = ""
        for e in m.embeds:
            fields = ""
            for f in e.fields:
                if f.inline:
                    cur_field = await fill_out(channel, msg_embed_field_inline, [
                        ("EMBED_FIELD_NAME", f.name),
                        ("EMBED_FIELD_VALUE", f.value),
                    ])
                    fields += cur_field
                else:
                    cur_field = await fill_out(channel, msg_embed_field, [
                        ("EMBED_FIELD_NAME", f.name),
                        ("EMBED_FIELD_VALUE", f.value),
                    ])
                    fields += cur_field

            # default values for embeds need explicit setting because
            # Embed.empty breaks just about everything
            title = e.title \
                if e.title != discord.Embed.Empty \
                else ""
            r, g, b = (e.colour.r, e.colour.g, e.colour.b) \
                if e.colour != discord.Embed.Empty \
                else (0x20, 0x22, 0x25)  # default colour
            desc = e.description \
                if e.description != discord.Embed.Empty \
                else ""
            author = e.author.name \
                if e.author.name != discord.Embed.Empty \
                else ""
            footer = e.footer.text \
                if e.footer.text != discord.Embed.Empty \
                else ""
            footer_icon = e.footer.icon_url \
                if e.footer.icon_url != discord.Embed.Empty \
                else None

            thumbnail = e.thumbnail.url \
                if e.thumbnail.url != discord.Embed.Empty \
                else ""

            image = e.image.url \
                if e.image.url != discord.Embed.Empty \
                else ""

            if image != "":
                image = await fill_out(channel, embed_image, [
                    ("EMBED_IMAGE", str(image)),
                ])

            if thumbnail != "":
                thumbnail = await fill_out(channel, embed_thumbnail, [
                    ("EMBED_THUMBNAIL", str(thumbnail)),
                ])

            footer_fields = ""
            if footer != "":
                if footer_icon:
                    cur_footer = await fill_out(channel, embed_footer_image, [
                        ("EMBED_FOOTER", footer),
                        ("EMBED_FOOTER_ICON", footer_icon)
                    ])
                else:
                    cur_footer = await fill_out(channel, embed_footer, [
                        ("EMBED_FOOTER", footer),
                    ])
                footer_fields += cur_footer

            cur_embed = await fill_out(channel, msg_embed, [
                ("EMBED_R", str(r)),
                ("EMBED_G", str(g)),
                ("EMBED_B", str(b)),
                ("EMBED_AUTHOR", author, PARSE_MODE_EMBED_EMOJI),
                ("EMBED_TITLE", title, PARSE_MODE_EMBED_EMOJI),
                ("EMBED_IMAGE", image),
                ("EMBED_THUMBNAIL", thumbnail),
                ("EMBED_DESC", desc, PARSE_MODE_EMBED),
                ("EMBED_FIELDS", fields, PARSE_MODE_EMBED_VALUE),
                ("EMBED_FOOTER", footer_fields, PARSE_MODE_EMBED_EMOJI)
            ])
            embeds += cur_embed

        attachments = ""

        for a in m.attachments:
            result_img = False
            for ending in img_types:
                if str(a.url).endswith(ending):
                    result_img = True

            if result_img:
                cur_attach = await fill_out(channel, img_attachment, [
                    ("ATTACH_URL", a.url),
                    ("ATTACH_URL_THUMB", a.url)
                ])
            else:
                file_mb = a.size / 1000000
                cur_attach = await fill_out(channel, msg_attachment, [
                    ("ATTACH_URL", a.url),
                    ("ATTACH_BYTES", str(file_mb)[:4] + "MB"),
                    ("ATTACH_FILE", str(a.filename))
                ])
            attachments += cur_attach

        if m.author.bot:
            ze_bot_tag = bot_tag
        else:
            ze_bot_tag = ""

        output = []
        if "http://" in m.content or "www." in m.content or "https://" in m.content:
            for word in m.content.split():
                if word.startswith("&lt;") and word.endswith("&gt;"):
                    pattern = r"&lt;(.*)&gt;"
                    url = re.search(pattern, word).group(1)
                    url = f'<a href="{url}">{url}</a>'
                    output.append(url)
                elif "http://" in word:
                    pattern = r"http://(.*)"
                    word_link = re.search(pattern, word).group(1)
                    word_full = f'<a href="http://{word_link}">http://{word_link}</a>'
                    word = re.sub(pattern, word_full, word)
                    output.append(word)
                elif "www." in word:
                    pattern = r"www.(.*)"
                    word_link = re.search(pattern, word).group(1)
                    word_full = f'<a href="www.{word_link}">www.{word_link}</a>'
                    word = re.sub(pattern, word_full, word)
                    output.append(word)
                elif "https://" in word:
                    pattern = r"https://(.*)"
                    word_link = re.search(pattern, word).group(1)
                    word_full = f'<a href="https://{word_link}">https://{word_link}</a>'
                    word = re.sub(pattern, word_full, word)
                    output.append(word)
                else:
                    output.append(word)
            m.content = " ".join(output)

        emojis = ""
        for reactions in m.reactions:
            react_emoji = reactions.emoji
            if react_emoji == "":
                emojis += ""
            elif ":" in str(react_emoji):
                emoji_animated = re.compile(r"&lt;a:.*:.*&gt;")
                if emoji_animated.search(str(react_emoji)):
                    file_ending = "gif"
                else:
                    file_ending = "png"
                pattern = r":.*:(\d*)"
                emoji_id = re.search(pattern, str(react_emoji)).group(1)
                cur_custom_emoji = await fill_out(channel, custom_emoji, [
                    ("EMOJI", str(emoji_id)),
                    ("EMOJI_COUNT", str(reactions.count)),
                    ("EMOJI_FILE", file_ending)
                ])
                emojis += cur_custom_emoji
            else:
                react_emoji = convert_emoji(react_emoji)
                cur_emoji = await fill_out(channel, emoji, [
                    ("EMOJI", str(react_emoji)),
                    ("EMOJI_COUNT", str(reactions.count))
                ])
                emojis += cur_emoji

        m.content = await parse_emoji(m.content)

        cur_msg = ""

        author_name = await escape_html(m.author.display_name)

        user_colour = await member_colour_translator(m.author)
        if previous_author == m.author.id and previous_timestamp > time_string:
            cur_msg = await fill_out(channel, continue_message, [
                ("AVATAR_URL", str(m.author.avatar_url)),
                ("NAME_TAG", "%s#%s" % (m.author.name, m.author.discriminator)),
                ("USER_ID", str(m.author.id)),
                ("NAME", str(author_name)),
                ("BOT_TAG", ze_bot_tag, PARSE_MODE_NONE),
                ("TIMESTAMP", time_string_final),
                ("MESSAGE_ID", str(m.id)),
                ("MESSAGE_CONTENT", m.content),
                ("EMBEDS", embeds, PARSE_MODE_NONE),
                ("ATTACHMENTS", attachments, PARSE_MODE_NONE),
                ("EMOJI", emojis)
            ])
        else:
            if previous_author != 0 and previous_timestamp != "":
                cur_msg = await fill_out(channel, end_message, [])
            cur_msg += await fill_out(channel, msg, [
                ("AVATAR_URL", str(m.author.avatar_url)),
                ("NAME_TAG", "%s#%s" % (m.author.name, m.author.discriminator)),
                ("USER_ID", str(m.author.id)),
                ("USER_COLOUR", user_colour),
                ("NAME", str(author_name)),
                ("BOT_TAG", ze_bot_tag, PARSE_MODE_NONE),
                ("TIMESTAMP", time_string_final),
                ("MESSAGE_ID", str(m.id)),
                ("MESSAGE_CONTENT", m.content),
                ("EMBEDS", embeds, PARSE_MODE_NONE),
                ("ATTACHMENTS", attachments, PARSE_MODE_NONE),
                ("EMOJI", emojis)
            ])
            previous_author = m.author.id
            previous_timestamp = time_string + timedelta(minutes=4)

        messages_html += cur_msg

    guild_icon = guild.icon_url
    if len(guild_icon) < 2:
        guild_icon = "https://discord.com/assets/dd4dbc0016779df1378e7812eabaa04d.png"
    guild_name = await escape_html(guild.name)
    transcript = await fill_out(channel, total, [
        ("SERVER_NAME", f"Guild: {guild_name}"),
        ("SERVER_AVATAR_URL", str(guild_icon), PARSE_MODE_NONE),
        ("CHANNEL_NAME", f"Channel: {channel.name}"),
        ("MESSAGE_COUNT", str(len(messages))),
        ("MESSAGES", messages_html, PARSE_MODE_NONE),
        ("TIMEZONE", str(eastern)),
    ])

    return transcript


PARSE_MODE_NONE = 0
PARSE_MODE_NO_MARKDOWN = 1
PARSE_MODE_MARKDOWN = 2
PARSE_MODE_EMBED = 3
PARSE_MODE_EMBED_VALUE = 4
PARSE_MODE_EMBED_EMOJI = 5


async def fill_out(channel, base, replacements):
    for r in replacements:
        if len(r) == 2:  # default case
            k, v = r
            r = (k, v, PARSE_MODE_MARKDOWN)

        k, v, mode = r

        if mode != PARSE_MODE_NONE:
            v = await escape_mentions(v)
            v = await escape_mentions(v)
            v = await unescape_mentions(v)
            v = await parse_mentions(v, channel.guild, bot)
        if mode == PARSE_MODE_MARKDOWN:
            v = await parse_markdown(v)
        if mode == PARSE_MODE_EMBED:
            v = await parse_embed_markdown(v)
            v = await parse_markdown(v)
            v = await parse_emoji(v)
        if mode == PARSE_MODE_EMBED_VALUE:
            v = await parse_embed_markdown(v)
            v = await parse_emoji(v)
        if mode == PARSE_MODE_EMBED_EMOJI:
            v = await parse_emoji(v)

        base = base.replace("{{" + k + "}}", v)

    return base


def read_file(filename):
    with open(filename, "r") as f:
        s = f.read()
    return s


img_types = [".png", ".jpeg", ".jpg", ".gif"]
total = read_file(dir_path + "/chat_exporter_html/base.html")
msg = read_file(dir_path + "/chat_exporter_html/message.html")
bot_tag = read_file(dir_path + "/chat_exporter_html/bot-tag.html")
msg_embed = read_file(dir_path + "/chat_exporter_html/message-embed.html")
msg_embed_field = read_file(dir_path + "/chat_exporter_html/message-embed-field.html")
msg_embed_field_inline = read_file(dir_path + "/chat_exporter_html/message-embed-field-inline.html")
img_attachment = read_file(dir_path + "/chat_exporter_html/image-attachment.html")
msg_attachment = read_file(dir_path + "/chat_exporter_html/message_attachment.html")
emoji = read_file(dir_path + "/chat_exporter_html/emoji_attachment.html")
custom_emoji = read_file(dir_path + "/chat_exporter_html/custom_emoji_attachment.html")
continue_message = read_file(dir_path + "/chat_exporter_html/continue_message.html")
end_message = read_file(dir_path + "/chat_exporter_html/end_message.html")
embed_footer = read_file(dir_path + "/chat_exporter_html/embed_footer.html")
embed_footer_image = read_file(dir_path + "/chat_exporter_html/embed_footer_image.html")
embed_image = read_file(dir_path + "/chat_exporter_html/embed_image.html")
embed_thumbnail = read_file(dir_path + "/chat_exporter_html/embed_thumbnail.html")
